@@ -54,7 +54,7 @@ int check_lx200ap_connection(int fd)
 {
     const struct timespec timeout = {0, 50000000L};
     int i = 0;
-    char temp_string[64];
+    char temp_string[256];
     int error_type;
     int nbytes_write = 0;
     int nbytes_read  = 0;
@@ -70,6 +70,7 @@ int check_lx200ap_connection(int fd)
     }
     for (i = 0; i < 2; i++)
     {
+        // This is the command to get the UTC offset. Used as a connection test.
         if ((error_type = tty_write_string(fd, "#:GG#", &nbytes_write)) != TTY_OK)
         {
             DEBUGFDEVICE(lx200ap_name, INDI::Logger::DBG_ERROR,
@@ -95,17 +96,22 @@ int check_lx200ap_connection(int fd)
 
     return -1;
 }
+
+// get UTC offset.
 int getAPUTCOffset(int fd, double *value)
 {
     int error_type;
     int nbytes_write = 0;
     int nbytes_read  = 0;
 
-    char temp_string[16];
+    char temp_string[256];
+    temp_string[0] = 0;
+    temp_string[1] = 0;
 
-    DEBUGFDEVICE(lx200ap_name, AP_DBG_SCOPE, "CMD <%s>", "#:GG#");
+    const char *cmd = "#:GG#";
+    DEBUGFDEVICE(lx200ap_name, AP_DBG_SCOPE, "CMD <%s>", cmd);
 
-    if ((error_type = tty_write_string(fd, "#:GG#", &nbytes_write)) != TTY_OK)
+    if ((error_type = tty_write_string(fd, cmd, &nbytes_write)) != TTY_OK)
         return error_type;
 
     if ((error_type = tty_read_section(fd, temp_string, '#', LX200_TIMEOUT, &nbytes_read)) != TTY_OK)
@@ -225,15 +231,19 @@ int getAPUTCOffset(int fd, double *value)
 int setAPObjectAZ(int fd, double az)
 {
     int h, m, s;
-    char temp_string[16];
+    char cmd[256];
+
+    // The azimuth should be 0-360.
+    while (az < 0) az += 360.0;
+    while (az > 360.0) az -= 360.0;
 
     getSexComponents(az, &h, &m, &s);
 
-    snprintf(temp_string, sizeof(temp_string), "#:Sz %03d*%02d:%02d#", h, m, s);
+    snprintf(cmd, sizeof(cmd), "#:Sz %03d*%02d:%02d#", h, m, s);
 
-    DEBUGFDEVICE(lx200ap_name, AP_DBG_SCOPE, "CMD <%s>", temp_string);
+    DEBUGFDEVICE(lx200ap_name, AP_DBG_SCOPE, "CMD <%s>", cmd);
 
-    return (setStandardProcedure(fd, temp_string));
+    return (setStandardProcedure(fd, cmd));
 }
 
 /* wildi Valid set Values are positive, add error condition */
@@ -241,37 +251,35 @@ int setAPObjectAZ(int fd, double az)
 int setAPObjectAlt(int fd, double alt)
 {
     int d, m, s;
-    char temp_string[16];
+    char cmd[256];
 
     getSexComponents(alt, &d, &m, &s);
+    if (d < 0) d = -d;
+    snprintf(cmd, sizeof(cmd), "#:Sa %s%02d*%02d:%02d#",
+             alt >= 0 ? "+" : "-", d, m, s);
 
-    /* case with negative zero */
-    if (!d && alt < 0)
-    {
-        snprintf(temp_string, sizeof(temp_string), "#:Sa -%02d*%02d:%02d#", d, m, s);
-    }
-    else
-    {
-        snprintf(temp_string, sizeof(temp_string), "#:Sa %+02d*%02d:%02d#", d, m, s);
-    }
+    DEBUGFDEVICE(lx200ap_name, AP_DBG_SCOPE, "CMD <%s>", cmd);
 
-    DEBUGFDEVICE(lx200ap_name, AP_DBG_SCOPE, "CMD <%s>", temp_string);
-
-    return (setStandardProcedure(fd, temp_string));
+    return (setStandardProcedure(fd, cmd));
 }
+
+// Set the UTC offset.
+// Previously this only set positive offsets.
+// Added the sign according to the doc in https://astro-physics.info/tech_support/mounts/protocol-cp3-cp4.pdf
 int setAPUTCOffset(int fd, double hours)
 {
     int h, m, s;
 
-    char temp_string[16];
+    char cmd[256];
 
     getSexComponents(hours, &h, &m, &s);
+    if (h < 0) h = -h;
+    snprintf(cmd, sizeof(cmd), "#:SG %s%02d:%02d:%02d#",
+             hours >= 0 ? "+" : "-", h, m, s);
 
-    snprintf(temp_string, sizeof(temp_string), "#:SG %+03d:%02d:%02d#", h, m, s);
+    DEBUGFDEVICE(lx200ap_name, AP_DBG_SCOPE, "CMD <%s>", cmd);
 
-    DEBUGFDEVICE(lx200ap_name, AP_DBG_SCOPE, "CMD <%s>", temp_string);
-
-    return (setStandardProcedure(fd, temp_string));
+    return (setStandardProcedure(fd, cmd));
 }
 int APSyncCM(int fd, char *matchedObject)
 {
@@ -280,9 +288,10 @@ int APSyncCM(int fd, char *matchedObject)
     int nbytes_write = 0;
     int nbytes_read  = 0;
 
-    DEBUGFDEVICE(lx200ap_name, AP_DBG_SCOPE, "CMD <%s>", "#:CM#");
+    const char *cmd = "#:CM#";
+    DEBUGFDEVICE(lx200ap_name, AP_DBG_SCOPE, "CMD <%s>", cmd);
 
-    if ((error_type = tty_write_string(fd, "#:CM#", &nbytes_write)) != TTY_OK)
+    if ((error_type = tty_write_string(fd, cmd, &nbytes_write)) != TTY_OK)
         return error_type;
 
     if ((error_type = tty_read_section(fd, matchedObject, '#', LX200_TIMEOUT, &nbytes_read)) != TTY_OK)
@@ -307,12 +316,12 @@ int APSyncCMR(int fd, char *matchedObject)
     int nbytes_write = 0;
     int nbytes_read  = 0;
 
-    DEBUGFDEVICE(lx200ap_name, AP_DBG_SCOPE, "CMD <%s>", "#:CMR#");
+    const char *cmd = "#:CMR#";
+    DEBUGFDEVICE(lx200ap_name, AP_DBG_SCOPE, "CMD <%s>", cmd);
 
-    if ((error_type = tty_write_string(fd, "#:CMR#", &nbytes_write)) != TTY_OK)
+    if ((error_type = tty_write_string(fd, cmd, &nbytes_write)) != TTY_OK)
         return error_type;
 
-    /* read_ret = portRead(matchedObject, -1, LX200_TIMEOUT); */
     if ((error_type = tty_read_section(fd, matchedObject, '#', LX200_TIMEOUT, &nbytes_read)) != TTY_OK)
         return error_type;
 
@@ -328,260 +337,156 @@ int APSyncCMR(int fd, char *matchedObject)
     return 0;
 }
 
-int selectAPPECState(int fd, int pecstate)
+
+int sendAPCommand(int fd, const char *cmd, const char *comment)
 {
     int error_type;
     int nbytes_write = 0;
 
+    DEBUGDEVICE(lx200ap_name, INDI::Logger::DBG_DEBUG, comment);
+    DEBUGFDEVICE(lx200ap_name, AP_DBG_SCOPE, "CMD <%s>", cmd);
+
+    if ((error_type = tty_write_string(fd, cmd, &nbytes_write)) != TTY_OK)
+        return error_type;
+    return 0;
+}
+
+int selectAPPECState(int fd, int pecstate)
+{
     switch (pecstate)
     {
-        // PEC OFF
-        case 0:
-            DEBUGDEVICE(lx200ap_name, INDI::Logger::DBG_DEBUG, "selectAPPECState: Setting PEC OFF");
-            DEBUGFDEVICE(lx200ap_name, AP_DBG_SCOPE, "CMD <%s>", "#:p#");
-
-            if ((error_type = tty_write_string(fd, "#:p#", &nbytes_write)) != TTY_OK)
-                return error_type;
-
-            break;
-
-        // PEC ON
-        case 1:
-            DEBUGDEVICE(lx200ap_name, INDI::Logger::DBG_DEBUG, "selectAPPECState: Setting PEC ON");
-            DEBUGFDEVICE(lx200ap_name, AP_DBG_SCOPE, "CMD <%s>", "#:pP#");
-
-            if ((error_type = tty_write_string(fd, "#:pP#", &nbytes_write)) != TTY_OK)
-                return error_type;
-
-            break;
-
+        case AP_PEC_OFF:
+            return sendAPCommand(fd, "#:p#", "selectAPPECState: Setting PEC OFF");
+        case AP_PEC_ON:
+            return sendAPCommand(fd, "#:pP#", "selectAPPECState: Setting PEC ON");
+        case AP_PEC_RECORD:
+            return sendAPCommand(fd, "#:pR#", "selectAPPECState: Enabling PEC RECORD");
         default:
             return -1;
-            break;
     }
 
     return 0;
 }
 
+
+// Should return a number between 0 and 969 inclusive.
+// It is a "normalized worm position", normalized to the number of PEM datapoints per revolution.
+// It is based on the gear angle and remembered through a power cycle.
+int getAPWormPosition(int fd, int *position)
+{
+    int nbytes_read  = 0;
+    char response[128];
+
+    int res = sendAPCommand(fd, "#:Gp#", "getAWormPosition");
+    if (res != TTY_OK)
+    {
+        DEBUGDEVICE(lx200ap_name, INDI::Logger::DBG_ERROR, "getAPWormPosition: write failed.");
+        return res;
+    }
+
+    res = tty_read_section(fd, response, '#', LX200_TIMEOUT, &nbytes_read);
+    if (res != TTY_OK)
+    {
+        // This does happen occasionally, not sure why, but isn't critical.
+        // DEBUGDEVICE(lx200ap_name, INDI::Logger::DBG_ERROR, "getAPWormPosition: read failed.");
+        return res;
+    }
+
+    tcflush(fd, TCIFLUSH);
+    if (nbytes_read > 1)
+    {
+        response[nbytes_read - 1] = '\0';
+        response[3] = '\0';
+        sscanf(response, "%d", position);
+        DEBUGFDEVICE(lx200ap_name, INDI::Logger::DBG_DEBUG, "getAPWormPosition: response: %d", *position);
+        return TTY_OK;
+    }
+    DEBUGDEVICE(lx200ap_name, INDI::Logger::DBG_ERROR, "getAPWormPosition: wrote, but bad response.");
+    return -1;
+}
+
 int selectAPMoveToRate(int fd, int moveToRate)
 {
-    int error_type;
-    int nbytes_write = 0;
-
     switch (moveToRate)
     {
-        /* 12x*/
         case 0:
-            DEBUGDEVICE(lx200ap_name, INDI::Logger::DBG_DEBUG, "selectAPMoveToRate: Setting move to rate to 12x");
-            DEBUGFDEVICE(lx200ap_name, AP_DBG_SCOPE, "CMD <%s>", "#:RC0#");
-
-            if ((error_type = tty_write_string(fd, "#:RC0#", &nbytes_write)) != TTY_OK)
-                return error_type;
-            break;
-
-        /* 64x */
+            return sendAPCommand(fd, "#:RC0#", "selectAPMoveToRate: Setting move to rate to 12x");
         case 1:
-            DEBUGDEVICE(lx200ap_name, INDI::Logger::DBG_DEBUG, "selectAPMoveToRate: Setting move to rate to 64x");
-            DEBUGFDEVICE(lx200ap_name, AP_DBG_SCOPE, "CMD <%s>", "#:RC1#");
-
-            if ((error_type = tty_write_string(fd, "#:RC1#", &nbytes_write)) != TTY_OK)
-                return error_type;
-            break;
-
-        /* 600x */
+            return sendAPCommand(fd, "#:RC1#", "selectAPMoveToRate: Setting move to rate to 64x");
         case 2:
-            DEBUGDEVICE(lx200ap_name, INDI::Logger::DBG_DEBUG, "selectAPMoveToRate: Setting move to rate to 600x");
-            DEBUGFDEVICE(lx200ap_name, AP_DBG_SCOPE, "CMD <%s>", "#:RC2#");
-            if ((error_type = tty_write_string(fd, "#:RC2#", &nbytes_write)) != TTY_OK)
-                return error_type;
-            break;
-
-        /* 1200x */
+            return sendAPCommand(fd, "#:RC2#", "selectAPMoveToRate: Setting move to rate to 600x");
         case 3:
-            DEBUGDEVICE(lx200ap_name, INDI::Logger::DBG_DEBUG, "selectAPMoveToRate: Setting move to rate to 1200x");
-            DEBUGFDEVICE(lx200ap_name, AP_DBG_SCOPE, "CMD <%s>", "#:RC3#");
-
-            if ((error_type = tty_write_string(fd, "#:RC3#", &nbytes_write)) != TTY_OK)
-                return error_type;
-            break;
-
+            return sendAPCommand(fd, "#:RC3#", "selectAPMoveToRate: Setting move to rate to 1200x");
         default:
             return -1;
-            break;
     }
     return 0;
 }
 
 int selectAPSlewRate(int fd, int slewRate)
 {
-    int error_type;
-    int nbytes_write = 0;
     switch (slewRate)
     {
-        /* 600x */
         case 0:
-
-            DEBUGDEVICE(lx200ap_name, INDI::Logger::DBG_DEBUG, "selectAPSlewRate: Setting slew to rate to 600x");
-            DEBUGFDEVICE(lx200ap_name, AP_DBG_SCOPE, "CMD <%s>", "#:RS0#");
-
-            if ((error_type = tty_write_string(fd, "#:RS0#", &nbytes_write)) != TTY_OK)
-                return error_type;
-            break;
-
-        /* 900x */
+            return sendAPCommand(fd, "#:RS0#", "selectAPSlewRate: Setting slew to rate to 600x");
         case 1:
-
-            DEBUGDEVICE(lx200ap_name, INDI::Logger::DBG_DEBUG, "selectAPSlewRate: Setting slew to rate to 900x");
-            DEBUGFDEVICE(lx200ap_name, AP_DBG_SCOPE, "CMD <%s>", "#:RS1#");
-
-            if ((error_type = tty_write_string(fd, "#:RS1#", &nbytes_write)) != TTY_OK)
-                return error_type;
-            break;
-
-
-        /* 1200x */
+            return sendAPCommand(fd, "#:RS1#", "selectAPSlewRate: Setting slew to rate to 900x");
         case 2:
-
-            DEBUGDEVICE(lx200ap_name, INDI::Logger::DBG_DEBUG, "selectAPSlewRate: Setting slew to rate to 1200x");
-            DEBUGFDEVICE(lx200ap_name, AP_DBG_SCOPE, "CMD <%s>", "#:RS2#");
-
-            if ((error_type = tty_write_string(fd, "#:RS2#", &nbytes_write)) != TTY_OK)
-                return error_type;
-            break;
-
+            return sendAPCommand(fd, "#:RS2#", "selectAPSlewRate: Setting slew to rate to 1200x");
         default:
             return -1;
-            break;
     }
     return 0;
 }
 
 int selectAPTrackingMode(int fd, int trackMode)
 {
-    int error_type;
-    int nbytes_write = 0;
-
     switch (trackMode)
     {
-        /* Sidereal */
         case AP_TRACKING_SIDEREAL:
-
-            DEBUGDEVICE(lx200ap_name, INDI::Logger::DBG_DEBUG,
-                        "selectAPTrackingMode: Setting tracking mode to sidereal.");
-            DEBUGFDEVICE(lx200ap_name, AP_DBG_SCOPE, "CMD <%s>", "#:RT2#");
-
-            if ((error_type = tty_write_string(fd, "#:RT2#", &nbytes_write)) != TTY_OK)
-                return error_type;
-            break;
-
-        /* Solar */
+            return sendAPCommand(fd, "#:RT2#", "selectAPTrackingMode: Setting tracking mode to sidereal.");
         case AP_TRACKING_SOLAR:
-
-            DEBUGDEVICE(lx200ap_name, INDI::Logger::DBG_DEBUG, "selectAPTrackingMode: Setting tracking mode to solar.");
-            DEBUGFDEVICE(lx200ap_name, AP_DBG_SCOPE, "CMD <%s>", "#:RT1#");
-
-            if ((error_type = tty_write_string(fd, "#:RT1#", &nbytes_write)) != TTY_OK)
-                return error_type;
-            break;
-
-        /* Lunar */
+            return sendAPCommand(fd, "#:RT1#", "selectAPTrackingMode: Setting tracking mode to solar.");
         case AP_TRACKING_LUNAR:
-
-            DEBUGDEVICE(lx200ap_name, INDI::Logger::DBG_DEBUG, "selectAPTrackingMode: Setting tracking mode to lunar.");
-            DEBUGFDEVICE(lx200ap_name, AP_DBG_SCOPE, "CMD <%s>", "#:RT0#");
-
-            if ((error_type = tty_write_string(fd, "#:RT0#", &nbytes_write)) != TTY_OK)
-                return error_type;
-            break;
-
+            return sendAPCommand(fd, "#:RT0#", "selectAPTrackingMode: Setting tracking mode to lunar.");
         case AP_TRACKING_CUSTOM:
-            DEBUGDEVICE(lx200ap_name, INDI::Logger::DBG_DEBUG, "selectAPTrackingMode: Setting tracking mode to Custom.");
+            DEBUGDEVICE(lx200ap_name, INDI::Logger::DBG_DEBUG,
+                        "selectAPTrackingMode: Setting tracking mode to Custom -- NOT IMPLEMENTED!.");
             break;
-
-        /* Zero */
         case AP_TRACKING_OFF:
-
-            DEBUGDEVICE(lx200ap_name, INDI::Logger::DBG_DEBUG, "selectAPTrackingMode: Setting tracking mode to Zero.");
-            DEBUGFDEVICE(lx200ap_name, AP_DBG_SCOPE, "CMD <%s>", "#:RT9#");
-
-            if ((error_type = tty_write_string(fd, "#:RT9#", &nbytes_write)) != TTY_OK)
-                return error_type;
-            break;
-
+            return sendAPCommand(fd, "#:RT9#", "selectAPTrackingMode: Setting tracking mode to Zero.");
         default:
             return -1;
-            break;
     }
     return 0;
 }
 
 int selectAPGuideRate(int fd, int guideRate)
 {
-    int error_type;
-    int nbytes_write = 0;
     switch (guideRate)
     {
-        /* 0.25x */
         case 0:
-
-            DEBUGDEVICE(lx200ap_name, INDI::Logger::DBG_DEBUG, "selectAPGuideRate: Setting guide to rate to 0.25x");
-            DEBUGFDEVICE(lx200ap_name, AP_DBG_SCOPE, "CMD <%s>", "#:RG0#");
-
-            if ((error_type = tty_write_string(fd, "#:RG0#", &nbytes_write)) != TTY_OK)
-                return error_type;
-            break;
-
-        /* 0.50x */
+            return sendAPCommand(fd, "#:RG0#", "selectAPGuideRate: Setting guide to rate to 0.25x");
         case 1:
-
-            DEBUGDEVICE(lx200ap_name, INDI::Logger::DBG_DEBUG, "selectAPGuideRate: Setting guide to rate to 0.50x");
-            DEBUGFDEVICE(lx200ap_name, AP_DBG_SCOPE, "CMD <%s>", "#:RG1#");
-
-            if ((error_type = tty_write_string(fd, "#:RG1#", &nbytes_write)) != TTY_OK)
-                return error_type;
-            break;
-
-
-        /* 1.00x */
+            return sendAPCommand(fd, "#:RG1#", "selectAPGuideRate: Setting guide to rate to 0.50x");
         case 2:
-
-            DEBUGDEVICE(lx200ap_name, INDI::Logger::DBG_DEBUG, "selectAPGuideRate: Setting guide to rate to 1.00x");
-            DEBUGFDEVICE(lx200ap_name, AP_DBG_SCOPE, "CMD <%s>", "#:RG2#");
-
-            if ((error_type = tty_write_string(fd, "#:RG2#", &nbytes_write)) != TTY_OK)
-                return error_type;
-            break;
-
+            return sendAPCommand(fd, "#:RG2#", "selectAPGuideRate: Setting guide to rate to 1.00x");
         default:
             return -1;
-            break;
     }
     return 0;
 }
 
 int swapAPButtons(int fd, int currentSwap)
 {
-    int error_type;
-    int nbytes_write = 0;
-
     switch (currentSwap)
     {
         case 0:
-
-            DEBUGFDEVICE(lx200ap_name, AP_DBG_SCOPE, "CMD <%s>", "#:NS#");
-            if ((error_type = tty_write_string(fd, "#:NS#", &nbytes_write)) != TTY_OK)
-                return error_type;
-            break;
-
+            return sendAPCommand(fd, "#:NS#", "swapAPButtons: Swapping NS.");
         case 1:
-            DEBUGFDEVICE(lx200ap_name, AP_DBG_SCOPE, "CMD <%s>", "#:EW#");
-            if ((error_type = tty_write_string(fd, "#:EW#", &nbytes_write)) != TTY_OK)
-                return error_type;
-            break;
-
+            return sendAPCommand(fd, "#:EW#", "swapAPButtons: Swapping EW.");
         default:
             return -1;
-            break;
     }
     return 0;
 }
@@ -590,62 +495,68 @@ int setAPObjectRA(int fd, double ra)
 {
     /*ToDo AP accepts "#:Sr %02d:%02d:%02d.%1d#"*/
     int h, m, s;
-    char temp_string[16];
+    char cmd[256];
+
+    // Make sure RA is 0-24.
+    while (ra < 0) ra += 24.0;
+    while (ra > 24.0) ra -= 24.0;
 
     getSexComponents(ra, &h, &m, &s);
 
-    snprintf(temp_string, sizeof(temp_string), "#:Sr %02d:%02d:%02d#", h, m, s);
+    snprintf(cmd, sizeof(cmd), "#:Sr %02d:%02d:%02d#", h, m, s);
 
-    DEBUGFDEVICE(lx200ap_name, AP_DBG_SCOPE, "CMD <%s>", temp_string);
+    DEBUGFDEVICE(lx200ap_name, AP_DBG_SCOPE, "CMD <%s>", cmd);
 
-    return (setStandardProcedure(fd, temp_string));
+    return (setStandardProcedure(fd, cmd));
 }
 
 int setAPObjectDEC(int fd, double dec)
 {
     int d, m, s;
-    char temp_string[16];
+    char cmd[256];
 
     getSexComponents(dec, &d, &m, &s);
-    /* case with negative zero */
-    if (!d && dec < 0)
-    {
-        snprintf(temp_string, sizeof(temp_string), "#:Sd -%02d*%02d:%02d#", d, m, s);
-    }
-    else
-    {
-        snprintf(temp_string, sizeof(temp_string), "#:Sd %+03d*%02d:%02d#", d, m, s);
-    }
+    if (d < 0) d = -d;
+    snprintf(cmd, sizeof(cmd), "#:Sd %s%02d*%02d:%02d#",
+             dec >= 0 ? "+" : "-", d, m, s);
 
-    DEBUGFDEVICE(lx200ap_name, AP_DBG_SCOPE, "CMD <%s>", temp_string);
+    DEBUGFDEVICE(lx200ap_name, AP_DBG_SCOPE, "CMD <%s>", cmd);
 
-    return (setStandardProcedure(fd, temp_string));
+    return (setStandardProcedure(fd, cmd));
 }
 
+// Set the longitude.
 int setAPSiteLongitude(int fd, double Long)
 {
     int d, m, s;
-    char temp_string[32];
+    char cmd[256];
+
+    // Make sure longitude is 0-360.
+    while (Long < 0) Long += 360.0;
+    while (Long > 360.0) Long -= 360.0;
 
     getSexComponents(Long, &d, &m, &s);
-    snprintf(temp_string, sizeof(temp_string), "#:Sg %03d*%02d:%02d#", d, m, s);
+    snprintf(cmd, sizeof(cmd), "#:Sg %03d*%02d:%02d#", d, m, s);
 
-    DEBUGFDEVICE(lx200ap_name, AP_DBG_SCOPE, "CMD <%s>", temp_string);
+    DEBUGFDEVICE(lx200ap_name, AP_DBG_SCOPE, "CMD <%s>", cmd);
 
-    return (setStandardProcedure(fd, temp_string));
+    return (setStandardProcedure(fd, cmd));
 }
 
+// Set the latitude.
 int setAPSiteLatitude(int fd, double Lat)
 {
     int d, m, s;
-    char temp_string[32];
+    char cmd[256];
 
     getSexComponents(Lat, &d, &m, &s);
-    snprintf(temp_string, sizeof(temp_string), "#:St %+03d*%02d:%02d#", d, m, s);
+    if (d < 0) d = -d;
+    snprintf(cmd, sizeof(cmd), "#:St %s%02d*%02d:%02d#",
+             Lat >= 0 ? "+" : "-", d, m, s);
 
-    DEBUGFDEVICE(lx200ap_name, AP_DBG_SCOPE, "CMD <%s>", temp_string);
+    DEBUGFDEVICE(lx200ap_name, AP_DBG_SCOPE, "CMD <%s>", cmd);
 
-    return (setStandardProcedure(fd, temp_string));
+    return (setStandardProcedure(fd, cmd));
 }
 
 int setAPRATrackRate(int fd, double rate)
@@ -663,7 +574,7 @@ int setAPRATrackRate(int fd, double rate)
     else
         sign = '+';
 
-    snprintf(cmd, 16, ":RR%c%03.4f#", sign, fabs(rate));
+    snprintf(cmd, 16, "#:RR%c%03.4f#", sign, fabs(rate));
 
     DEBUGFDEVICE(lx200ap_name, INDI::Logger::DBG_DEBUG, "CMD (%s)", cmd);
 
@@ -711,7 +622,7 @@ int setAPDETrackRate(int fd, double rate)
     else
         sign = '+';
 
-    snprintf(cmd, 16, ":RD%c%03.4f#", sign, fabs(rate));
+    snprintf(cmd, 16, "#:RD%c%03.4f#", sign, fabs(rate));
 
     DEBUGFDEVICE(lx200ap_name, INDI::Logger::DBG_DEBUG, "CMD (%s)", cmd);
 
@@ -747,8 +658,6 @@ int setAPDETrackRate(int fd, double rate)
 
 int APSendPulseCmd(int fd, int direction, int duration_msec)
 {
-    DEBUGFDEVICE(lx200ap_name, AP_DBG_SCOPE, "<%s>", __FUNCTION__);
-    int nbytes_write = 0;
     char cmd[20];
 
     // GTOCP3 supports 3 digits for msec duration
@@ -761,105 +670,35 @@ int APSendPulseCmd(int fd, int direction, int duration_msec)
     switch (direction)
     {
         case LX200_NORTH:
-            sprintf(cmd, ":Mn%03d#", duration_msec);
+            sprintf(cmd, "#:Mn%03d#", duration_msec);
             break;
         case LX200_SOUTH:
-            sprintf(cmd, ":Ms%03d#", duration_msec);
+            sprintf(cmd, "#:Ms%03d#", duration_msec);
             break;
         case LX200_EAST:
-            sprintf(cmd, ":Me%03d#", duration_msec);
+            sprintf(cmd, "#:Me%03d#", duration_msec);
             break;
         case LX200_WEST:
-            sprintf(cmd, ":Mw%03d#", duration_msec);
+            sprintf(cmd, "#:Mw%03d#", duration_msec);
             break;
         default:
             return 1;
     }
 
-    DEBUGFDEVICE(lx200ap_name, AP_DBG_SCOPE, "CMD <%s>", cmd);
-
-    tty_write_string(fd, cmd, &nbytes_write);
-
+    int res = sendAPCommand(fd, cmd, "APSendPulseCmd: Sending pulse command.");
     tcflush(fd, TCIFLUSH);
-    return 0;
+    return res;
 }
 
-#if 0
-// experimental function!!!
-int check_lx200ap_status(int fd, char *parkStatus, char *slewStatus)
-{
-    char temp_string[64];
-    int error_type;
-    int nbytes_write = 0;
-    int nbytes_read  = 0;
-
-    DEBUGDEVICE(lx200ap_name, INDI::Logger::DBG_DEBUG, "EXPERIMENTAL: check status...");
-
-    if (fd <= 0)
-    {
-        DEBUGDEVICE(lx200ap_name, INDI::Logger::DBG_ERROR,
-                    "check_lx200ap_connection: not a valid file descriptor received");
-
-        return -1;
-    }
-
-    if ((error_type = tty_write_string(fd, "#:GOS#", &nbytes_write)) != TTY_OK)
-    {
-        DEBUGFDEVICE(lx200ap_name, INDI::Logger::DBG_ERROR,
-                     "check_lx200ap_connection: unsuccessful write to telescope, %d", nbytes_write);
-
-        return error_type;
-    }
-    tty_read_section(fd, temp_string, '#', LX200_TIMEOUT, &nbytes_read);
-    tcflush(fd, TCIFLUSH);
-    if (nbytes_read > 1)
-    {
-        temp_string[nbytes_read - 1] = '\0';
-
-        DEBUGFDEVICE(lx200ap_name, INDI::Logger::DBG_DEBUG, "check_lx200ap_status: received bytes %d, [%s]",
-                     nbytes_write, temp_string);
-
-        *parkStatus = temp_string[0];
-        *slewStatus = temp_string[3];
-
-        return 0;
-    }
-
-
-    DEBUGDEVICE(lx200ap_name, INDI::Logger::DBG_ERROR, "check_lx200ap_status: wrote, but nothing received.");
-
-    return -1;
-}
-#endif
-
-// make this a function with logging instead of a #define like in legacy driver
 int APParkMount(int fd)
 {
-    int error_type;
-    int nbytes_write = 0;
-
-    DEBUGDEVICE(lx200ap_name, INDI::Logger::DBG_DEBUG, "APParkMount: Sending park command.");
-    DEBUGFDEVICE(lx200ap_name, AP_DBG_SCOPE, "CMD <%s>", "#:KA");
-
-    if ((error_type = tty_write_string(fd, "#:KA", &nbytes_write)) != TTY_OK)
-        return error_type;
-
-    return 0;
+    return sendAPCommand(fd, "#:KA#", "APParkMount: Sending park command.");
 }
 
-// make this a function with logging instead of a #define like in legacy driver
+
 int APUnParkMount(int fd)
 {
-    int error_type;
-    int nbytes_write = 0;
-
-    DEBUGDEVICE(lx200ap_name, INDI::Logger::DBG_DEBUG, "APUnParkMount: Sending unpark command.");
-    DEBUGFDEVICE(lx200ap_name, AP_DBG_SCOPE, "CMD <%s>", "#:PO");
-
-    if ((error_type = tty_write_string(fd, "#:PO", &nbytes_write)) != TTY_OK)
-        return error_type;
-
-    return 0;
+    return sendAPCommand(fd, "#:PO#", "APUnParkMount: Sending unpark command.");
 }
 
 // This is a modified version of selectAPMoveRate() from lx200apdriver.cpp
@@ -869,105 +708,229 @@ int APUnParkMount(int fd)
 // to GUIDE and then starting and halting a move of the correct duration.
 int selectAPCenterRate(int fd, int centerRate)
 {
-    int error_type;
-    int nbytes_write = 0;
-
     switch (centerRate)
     {
-        /* Guide */
         case 0:
-            DEBUGDEVICE(lx200ap_name, INDI::Logger::DBG_DEBUG, "selectAPMoveToRate: Setting move to rate to GUIDE");
-            DEBUGFDEVICE(lx200ap_name, AP_DBG_SCOPE, "CMD <%s>", "#:RG#");
-
-            if ((error_type = tty_write_string(fd, "#:RG#", &nbytes_write)) != TTY_OK)
-                return error_type;
-            break;
-
-        /* 12x */
+            return sendAPCommand(fd, "#:RG#", "selectAPMoveToRate: Setting move to rate to GUIDE");
         case 1:
-            DEBUGDEVICE(lx200ap_name, INDI::Logger::DBG_DEBUG, "selectAPMoveToRate: Setting move to rate to 12x");
-            DEBUGFDEVICE(lx200ap_name, AP_DBG_SCOPE, "CMD <%s>", "#:RC0#");
-
-            if ((error_type = tty_write_string(fd, "#:RC0#", &nbytes_write)) != TTY_OK)
-                return error_type;
-            break;
-
-        /* 64x */
+            return sendAPCommand(fd, "#:RC0#", "selectAPMoveToRate: Setting move to rate to 12x");
         case 2:
-            DEBUGDEVICE(lx200ap_name, INDI::Logger::DBG_DEBUG, "selectAPMoveToRate: Setting move to rate to 64x");
-            DEBUGFDEVICE(lx200ap_name, AP_DBG_SCOPE, "CMD <%s>", "#:RC1#");
-
-            if ((error_type = tty_write_string(fd, "#:RC1#", &nbytes_write)) != TTY_OK)
-                return error_type;
-            break;
-
-        /* 600x */
+            return sendAPCommand(fd, "#:RC1#", "selectAPMoveToRate: Setting move to rate to 64x");
         case 3:
-            DEBUGDEVICE(lx200ap_name, INDI::Logger::DBG_DEBUG, "selectAPMoveToRate: Setting move to rate to 600x");
-            DEBUGFDEVICE(lx200ap_name, AP_DBG_SCOPE, "CMD <%s>", "#:RC2#");
-            if ((error_type = tty_write_string(fd, "#:RC2#", &nbytes_write)) != TTY_OK)
-                return error_type;
-            break;
-
-        /* 1200x */
+            return sendAPCommand(fd, "#:RC2#", "selectAPMoveToRate: Setting move to rate to 600x");
         case 4:
-            DEBUGDEVICE(lx200ap_name, INDI::Logger::DBG_DEBUG, "selectAPMoveToRate: Setting move to rate to 1200x");
-            DEBUGFDEVICE(lx200ap_name, AP_DBG_SCOPE, "CMD <%s>", "#:RC3#");
-
-            if ((error_type = tty_write_string(fd, "#:RC3#", &nbytes_write)) != TTY_OK)
-                return error_type;
-            break;
-
+            return sendAPCommand(fd, "#:RC3#", "selectAPMoveToRate: Setting move to rate to 1200x");
         default:
             return -1;
-            break;
     }
     return 0;
 }
 
-// experimental functions!!!
+// Doc for the :GOS command fom A-P:
+//
+// Response for GTOCP3 Rev “T” through GTOCP4 Rev VCP4-P01-14 is a 13 character string: ABCDEFGHIJKLM.
+//
+// Note the addition of the last two characters: L & M. (Rev “S” had the 1st 11 characters)
+// 14th character “N” added in VCPx-P02-xx and later
+//
+// Possible values for each variable are as follows (Please note the difference between “0” and “O” in the responses)
+// Note the differences starting with the GTOCP4, especially with P02-01 and later!!
+//
+// A: Park Status                         'P' parked, '0' not parked, '1' auto-park - ON
+// B: RA Tracking Status                  '0'=Lunar Rate, '1'=Solar Rate, '2'=Sidereal, '9'=Tracking Stopped,
+//                                        'C'=Custom RA Tracking Rate (read specific value with :Rr# command)
+// C: Dec Tracking Status                 '9'=No Motion (to mimic RA Tracking), 'C'=Custom DEC Tracking Rate
+//                                        (read specific value with :Rd# command)
+// D: Slewing Satus (GOTO Slews)          'S'=Slewing, '0'=Not slewing
+// E: Moving RA Axis                      'E'=Moving East, 'W'=Moving West, '0'=Not Moving
+//    (via a Move command/Slew/ST4 Port signal)
+// F: Moving Dec Axis                     'N'=Moving North (counter-clockwise), 'S'=Moving South (clockwise), '0'=Not Moving
+//    (via a Move command/Slew/ST4 Port signal):
+// G: Guide Rate                          '0'=0.25x, '1'=0.50x, '2'=1.00x
+// H: Center/Move Rate < P02-xx           '0'=12x, '1'=64x, '2'=600x, '3'=1200x, 'C'=Custom Rate
+//                                        NOTE: Divide rates for '2' and '3' by 2 for 3600GTO
+//                                        Read “C” rate with  :Rc#  command.  Do not divide by 2 for 3600GTO
+// H: Center/Move Rate > P02-xx           '0'=12x, '1'=64x, '2'=200x, '3'= 400x - 600x  see table,
+//                                        '4'= 600x - 1200x  see table, '5'=0.25x, '6'=0.5x, '7'=1.0x, 'C'=Custom Rate
+// I: Slew Rate                           '0'=600x     Slow, '1'=900x     Medium, '2'=1200x   Fast, 'C'=Custom Rate
+//                                        NOTE: Rates are scaled for “0”, “1” & “2” by different amounts for 3600GTOs
+//                                        and for some 400GTO and 600EGTO mounts.
+//                                        Read 'C' rate with  :Rs#  command.  Do not scale custom rates.
+// J: PEM                                'O'=Off, 'P'=Playback, 'R'=Recording, 'E'=Encoder
+// K: Mount Status                       '0'=Normal, '1'=Stalled, '2'=Low Power Supply,
+//                                       '4'=Servo fault / number problem, '8'=Reserved  (CP3 only)
+// K: Mount Status                       '0'=Normal 'Z'=Stalled, 'Y'=Low Power Supply, 'X'=Servo fault / number problem,
+//    VCP4-P02-01 and later              'N'=CCW Internal Declination Limit or AE Limit, 'S'=CW Internal Declination Limit or AE Limit,
+//                                       'E'=East Internal RA Limit or AE Limit, 'W'=West Internal RA Limit or AE Limit,
+//                                       'z'=Kill Function has been issued
+// L: E – W button reversal              '0'=Normal  E = E  and W = W, '1'=Reversed   E = W  and  W = E
+// M: N – S button reversal              '0'=Normal   N = CCW  and  S = CW, '1'=Reversed    N = CW  and  S = CCW
+// N: Button / Slew Rate Table           '0'=Normal, '1'= ~75% speed reduction, '2'= ~50% speed reduction, '3'= High Speed for Mach2GTO
+//    VCP4-P02-01 and later
+//
 
-
-int check_lx200ap_status(int fd, char *parkStatus, char *slewStatus)
+int getApStatusStringInternal(int fd, char *statusString, bool complain)
 {
-    char temp_string[64];
-    int error_type;
     int nbytes_write = 0;
     int nbytes_read  = 0;
 
-    DEBUGDEVICE(lx200ap_name, INDI::Logger::DBG_DEBUG, "Check status...");
-
     if (fd <= 0)
     {
-        DEBUGDEVICE(lx200ap_name, INDI::Logger::DBG_ERROR,
-                    "check_lx200ap_connection: not a valid file descriptor received");
-
+        if (complain) DEBUGDEVICE(lx200ap_name, INDI::Logger::DBG_ERROR,
+                                      "getApStatusString: not a valid file descriptor received");
         return -1;
     }
 
-    if ((error_type = tty_write_string(fd, "#:GOS#", &nbytes_write)) != TTY_OK)
+    int res = sendAPCommand(fd, "#:GOS#", "getApStatusString");
+    if (res != TTY_OK)
     {
-        DEBUGFDEVICE(lx200ap_name, INDI::Logger::DBG_ERROR,
-                     "check_lx200ap_connection: unsuccessful write to telescope, %d", nbytes_write);
-
-        return error_type;
+        if (complain) DEBUGFDEVICE(lx200ap_name, INDI::Logger::DBG_ERROR,
+                                       "getApStatusString: unsuccessful write to telescope, %d", nbytes_write);
+        return res;
     }
-    tty_read_section(fd, temp_string, '#', LX200_TIMEOUT, &nbytes_read);
+
+    tty_read_section(fd, statusString, '#', LX200_TIMEOUT, &nbytes_read);
     tcflush(fd, TCIFLUSH);
-    if (nbytes_read > 1)
+    if (nbytes_read > 3)
     {
-        temp_string[nbytes_read - 1] = '\0';
+        statusString[nbytes_read - 1] = '\0';
 
-        DEBUGFDEVICE(lx200ap_name, INDI::Logger::DBG_DEBUG, "check_lx200ap_status: received bytes %d, [%s]",
-                     nbytes_write, temp_string);
-
-        *parkStatus = temp_string[0];
-        *slewStatus = temp_string[3];
+        DEBUGFDEVICE(lx200ap_name, INDI::Logger::DBG_DEBUG, "getApStatusString: received bytes %d, [%s]",
+                     nbytes_write, statusString);
 
         return 0;
     }
 
-    DEBUGDEVICE(lx200ap_name, INDI::Logger::DBG_ERROR, "check_lx200ap_status: wrote, but nothing received.");
+    if (complain) DEBUGDEVICE(lx200ap_name, INDI::Logger::DBG_ERROR, "getApStatusString: wrote, but nothing received.");
 
     return -1;
 }
+
+int getApStatusString(int fd, char *statusString)
+{
+    // I seem to get intermittant failures.
+    // Try again on these after a 50ms delay, and the 250ms delay.
+    if (getApStatusStringInternal(fd, statusString, false) != TTY_OK)
+    {
+        const struct timespec timeout50ms = {0, 50000000L};
+        nanosleep(&timeout50ms, nullptr);
+        if (getApStatusStringInternal(fd, statusString, true) == TTY_OK)
+            return TTY_OK;
+        else
+        {
+            const struct timespec timeout250ms = {0, 250000000L};
+            nanosleep(&timeout250ms, nullptr);
+            return getApStatusStringInternal(fd, statusString, true);
+        }
+    }
+    return TTY_OK;
+}
+
+int check_lx200ap_status(int fd, char *parkStatus, char *slewStatus)
+{
+    char status_string[256];
+    int res = getApStatusString(fd, status_string);
+    if (res != TTY_OK) return res;
+
+    *parkStatus = status_string[0];
+    *slewStatus = status_string[3];
+    return TTY_OK;
+}
+
+// See above for the full list.
+// K: Mount Status                       '0'=Normal, '1'=Stalled, '2'=Low Power Supply,
+//                                       '4'=Servo fault / number problem, '8'=Reserved  (CP3 only)
+// K: Mount Status                       '0'=Normal 'Z'=Stalled, 'Y'=Low Power Supply, 'X'=Servo fault / number problem,
+//    VCP4-P02-01 and later              'N'=CCW Internal Declination Limit or AE Limit, 'S'=CW Internal Declination Limit or AE Limit,
+//                                       'E'=East Internal RA Limit or AE Limit, 'W'=West Internal RA Limit or AE Limit,
+//                                       'z'=Kill Function has been issued
+const char *apMountStatus(const char *statusString)
+{
+    if (strlen(statusString) < 11)
+        return "????";
+    const char statusChar = statusString[10];
+    switch (statusChar)
+    {
+        case '0':
+            return "Normal";
+        case '1':
+        case 'Z':
+            return "Stalled";
+        case '2':
+        case 'Y':
+            return "Low Power Supply";
+        case '4':
+        case 'X':
+            return "Servo Fault";
+        case 'N':
+            return "CCW DEC or AE Limit";
+        case 'S':
+            return "CW DEC or AE Limit";
+        case 'E':
+            return "East RA or AE Limit";
+        case 'W':
+            return "West RA or AE Limit";
+        case 'z':
+            return "Kill Function issued";
+        case '8':
+        default:
+            return "";
+    }
+}
+
+bool apStatusParked(const char *statusString)
+{
+    return statusString[0] == 'P';
+}
+
+bool apStatusSlewing(const char *statusString)
+{
+    return statusString[3] !=  '0';
+}
+int isAPInitialized(int fd, bool *isInitialized)
+{
+    constexpr int RB_MAX_LEN = 256;
+    char readBuffer[RB_MAX_LEN];
+    int error_type;
+    int nbytes_write = 0;
+    int nbytes_read  = 0;
+
+    DEBUGDEVICE(lx200ap_name, INDI::Logger::DBG_DEBUG, "Check initialized...");
+
+    if (fd <= 0)
+    {
+        DEBUGDEVICE(lx200ap_name, INDI::Logger::DBG_ERROR,
+                    "isAPInitialized: not a valid file descriptor received");
+
+        return -1;
+    }
+
+    if ((error_type = tty_write_string(fd, "#:GR#", &nbytes_write)) != TTY_OK)
+    {
+        DEBUGFDEVICE(lx200ap_name, INDI::Logger::DBG_ERROR,
+                     "isAPInitialized: unsuccessful write to telescope, %d", nbytes_write);
+
+        return error_type;
+    }
+
+    error_type = tty_nread_section(fd, readBuffer, RB_MAX_LEN, '#', LX200_TIMEOUT, &nbytes_read);
+
+
+    if (nbytes_read < 1)
+    {
+        DEBUGFDEVICE(lx200ap_name, INDI::Logger::DBG_DEBUG, "RES ERROR <%d>", error_type);
+        return error_type;
+    }
+
+    readBuffer[nbytes_read - 1] = '\0';
+
+    if (!strcmp("00:00.0", readBuffer))
+        *isInitialized = false;
+    else if (!strcmp("00:00:00.0", readBuffer))
+        *isInitialized = false; // not sure about this one--high precision 0.
+    else
+        *isInitialized = true; // Should I test further????
+
+    tcflush(fd, TCIFLUSH);
+    return 0;
+}
