@@ -33,6 +33,7 @@ Updated driver to use INDI::Telescope (JM)
 #include "lx200_16.h"
 #include "lx200_OnStep.h"
 #include "lx200ap.h"
+#include "lx200ap_v2.h"
 #include "lx200ap_gtocp2.h"
 #include "lx200classic.h"
 #include "lx200fs2.h"
@@ -65,6 +66,8 @@ static class Loader
     public:
         Loader()
         {
+            // Note: these if statements use strstr() which isn't a full string match, just a substring,
+            // so if one driver name is the start of another's name, it needs to be AFTER the longer one!
             if (strstr(__progname, "indi_lx200classic"))
             {
                 IDLog("initializing from LX200 classic device...\n");
@@ -90,16 +93,21 @@ static class Loader
                 IDLog("initializing from Autostar device...\n");
                 telescope.reset(new LX200Autostar());
             }
-            else if (strstr(__progname, "indi_lx200ap"))
+            else if (strstr(__progname, "indi_lx200ap_v2"))
             {
-                IDLog("initializing from Astrophysics device...\n");
-                telescope.reset(new LX200AstroPhysics());
+                IDLog("initializing from Astrophysics V2 device...\n");
+                telescope.reset(new LX200AstroPhysicsV2());
             }
             else if (strstr(__progname, "indi_lx200ap_gtocp2"))
             {
                 IDLog("initializing from Astrophysics GTOCP2 device...\n");
                 telescope.reset(new LX200AstroPhysicsGTOCP2());
             }
+            else if (strstr(__progname, "indi_lx200ap"))
+            {
+                IDLog("initializing from Astrophysics device...\n");
+                telescope.reset(new LX200AstroPhysics());
+            }
             else if (strstr(__progname, "indi_lx200gemini"))
             {
                 IDLog("initializing from Losmandy Gemini device...\n");
